@@ -1,11 +1,11 @@
 // Menu / panel border size
-const MARGIN = 4;
+var MARGIN = 4;
 
 // Includes the main menu button
-const NUM_BUTTONS = 9;
+const NUM_BUTTONS = 10;
 
 // Regular menu buttons
-const BUTTON_HEIGHT = 32;
+var BUTTON_HEIGHT = 32;
 
 // The position where the bottom of the menu will end up (currently 228)
 const END_MENU_POSITION = (BUTTON_HEIGHT * NUM_BUTTONS) + MARGIN;
@@ -16,17 +16,22 @@ const MENU_BOTTOM = 0;
 // Menu starting position: top
 const MENU_TOP = MENU_BOTTOM - END_MENU_POSITION;
 
-// Menu starting position: overall
-const INITIAL_MENU_POSITION = "100%-164 " + MENU_TOP + " 100% " + MENU_BOTTOM;
-
 // Number of pixels per millisecond to move
-const MENU_SPEED = 1.2;
+var MENU_SPEED = 1.2;
 
 // Trade menu: step for probability changes
-const STEP = 5;
+var STEP = 5;
 
 // Shown in the trade dialog.
-const g_IdleTraderTextColor = "orange";
+var g_IdleTraderTextColor = "orange";
+
+/**
+ * Store civilization code and page (structree or history) opened in civilization info.
+ */
+var g_CivInfo = {
+	"code": "",
+	"page": "page_structree.xml"
+};
 
 /**
  * The barter constants should match with the simulation
@@ -55,13 +60,27 @@ var g_IsDiplomacyOpen = false;
 var g_IsTradeOpen = false;
 var g_IsObjectivesOpen = false;
 
+/**
+ * Used to disable a specific bribe button for the time we are waiting for the result of the bribe after it was clicked.
+ * It contains an array per viewedPlayer. This array is a list of the players that were bribed.
+ */
+var g_BribeButtonsWaiting = {};
+
+/**
+ * Remember last viewed summary panel and charts.
+ */
+var g_SummarySelectedData;
+
 // Redefined every time someone makes a tribute (so we can save some data in a closure). Called in input.js handleInputBeforeGui.
 var g_FlushTributing = function() {};
 
-// Ignore size defined in XML and set the actual menu size here
-function initMenuPosition()
+function initMenu()
 {
-	Engine.GetGUIObjectByName("menu").size = INITIAL_MENU_POSITION;
+	Engine.GetGUIObjectByName("menu").size = "100%-164 " + MENU_TOP + " 100% " + MENU_BOTTOM;
+
+	// TODO: Atlas should pass g_GameAttributes.settings
+	for (let button of ["menuExitButton", "summaryButton", "objectivesButton", "diplomacyButton"])
+		Engine.GetGUIObjectByName(button).enabled = !Engine.IsAtlasRunning();
 }
 
 function updateMenuPosition(dt)
@@ -106,6 +125,15 @@ function optionsMenuButton()
 	openOptions();
 }
 
+function lobbyDialogButton()
+{
+	if (!Engine.HasXmppClient())
+		return;
+
+	closeOpenDialogs();
+	Engine.PushGuiPage("page_lobby.xml", { "dialog": true });
+}
+
 function chatMenuButton()
 {
 	closeOpenDialogs();
@@ -185,7 +213,7 @@ function openDeleteDialog(selection)
 {
 	closeOpenDialogs();
 
-	let deleteSelectedEntities = function (selectionArg)
+	let deleteSelectedEntities = function(selectionArg)
 	{
 		Engine.PostNetworkCommand({
 			"type": "delete-entities",
@@ -220,10 +248,19 @@ function openOptions()
 	pauseGame();
 
 	Engine.PushGuiPage("page_options.xml", {
-		"callback": "resumeGame"
+		"callback": "optionsPageClosed"
 	});
 }
 
+function optionsPageClosed(data)
+{
+	for (let callback of data)
+		if (global[callback])
+			global[callback]();
+
+	resumeGame();
+}
+
 function openChat(command = "")
 {
 	if (g_Disconnected)
@@ -251,13 +288,17 @@ function resizeDiplomacyDialog()
 {
 	let dialog = Engine.GetGUIObjectByName("diplomacyDialogPanel");
 	let size = dialog.size;
-	let width = size.right - size.left;
 
 	let tribSize = Engine.GetGUIObjectByName("diplomacyPlayer[0]_tribute[0]").size;
-	width += g_ResourceData.GetCodes().length * (tribSize.right - tribSize.left);
+	let widthOffset = g_ResourceData.GetCodes().length * (tribSize.right - tribSize.left) / 2;
+	size.left -= widthOffset;
+	size.right += widthOffset;
+
+	let firstRow = Engine.GetGUIObjectByName("diplomacyPlayer[0]").size;
+	let heightOffset = (g_Players.length - 1) * (firstRow.bottom - firstRow.top) / 2;
+	size.top -= heightOffset;
+	size.bottom += heightOffset;
 
-	size.left = -width / 2;
-	size.right = width / 2;
 	dialog.size = size;
 }
 
@@ -292,8 +333,8 @@ function resizeChatWindow()
 		let height = chatHistory.getComputedSize().bottom - chatHistory.getComputedSize().top;
 		let width = (1 + Math.sqrt(5)) / 2 * height;
 		let size = chatDialogPanel.size;
-		size.left = -width/2 - chatHistory.size.left;
-		size.right = width/2 + chatHistory.size.left;
+		size.left = -width / 2 - chatHistory.size.left;
+		size.right = width / 2 + chatHistory.size.left;
 		chatDialogPanel.size = size;
 	}
 	else
@@ -322,10 +363,7 @@ function updateChatHistory()
 
 function onToggleChatWindowExtended()
 {
-	// Save user preference
-	let extended = Engine.GetGUIObjectByName("extendedChat").checked.toString();
-	Engine.ConfigDB_CreateValue("user", "chat.session.extended", extended);
-	Engine.ConfigDB_WriteValueToFile("user", "chat.session.extended", extended, "config/user.cfg");
+	saveSettingAndWriteToUserConfig("chat.session.extended", String(Engine.GetGUIObjectByName("extendedChat").checked));
 
 	resizeChatWindow();
 
@@ -392,7 +430,7 @@ function updateDiplomacy(opening = false)
 
 	let diplomacyCeasefireCounter = Engine.GetGUIObjectByName("diplomacyCeasefireCounter");
 	diplomacyCeasefireCounter.caption = sprintf(
-		translateWithContext("ceasefire", "Time remaining until ceasefire is over: %(time)s."),
+		translateWithContext("ceasefire", "Remaining ceasefire time: %(time)s."),
 		{ "time": timeToString(simState.ceasefireTimeRemaining) }
 	);
 	diplomacyCeasefireCounter.hidden = !isCeasefireActive;
@@ -401,34 +439,38 @@ function updateDiplomacy(opening = false)
 function diplomacySetupTexts(i, rowsize)
 {
 	// Apply offset
-	let row = Engine.GetGUIObjectByName("diplomacyPlayer["+(i-1)+"]");
+	let row = Engine.GetGUIObjectByName("diplomacyPlayer[" + (i - 1) + "]");
 	let size = row.size;
-	size.top = rowsize * (i-1);
+	size.top = rowsize * (i - 1);
 	size.bottom = rowsize * i;
 	row.size = size;
 	row.hidden = false;
 
-	row.sprite = "color: " + rgbToGuiColor(g_Players[i].color) + " 32";
+	row.sprite = "color:" + rgbToGuiColor(g_DisplayedPlayerColors[i], 32);
 
-	setOutcomeIcon(g_Players[i].state, "diplomacyPlayerOutcome["+(i-1)+"]");
+	setOutcomeIcon(g_Players[i].state, "diplomacyPlayerOutcome[" + (i - 1) + "]");
 
-	Engine.GetGUIObjectByName("diplomacyPlayerName["+(i-1)+"]").caption = colorizePlayernameByID(i);
-	Engine.GetGUIObjectByName("diplomacyPlayerCiv["+(i-1)+"]").caption = g_CivData[g_Players[i].civ].Name;
+	let diplomacyPlayerName = Engine.GetGUIObjectByName("diplomacyPlayerName[" + (i - 1) + "]");
+	diplomacyPlayerName.caption = colorizePlayernameByID(i);
+	diplomacyPlayerName.tooltip = translateAISettings(g_GameAttributes.settings.PlayerData[i]);
 
-	Engine.GetGUIObjectByName("diplomacyPlayerTeam["+(i-1)+"]").caption =
-		g_Players[i].team < 0 ? translateWithContext("team", "None") : g_Players[i].team+1;
+	Engine.GetGUIObjectByName("diplomacyPlayerCiv[" + (i - 1) + "]").caption = g_CivData[g_Players[i].civ].Name;
 
-	Engine.GetGUIObjectByName("diplomacyPlayerTheirs["+(i-1)+"]").caption =
+	Engine.GetGUIObjectByName("diplomacyPlayerTeam[" + (i - 1) + "]").caption =
+		g_Players[i].team < 0 ? translateWithContext("team", "None") : g_Players[i].team + 1;
+
+	Engine.GetGUIObjectByName("diplomacyPlayerTheirs[" + (i - 1) + "]").caption =
 		i == g_ViewedPlayer ? "" :
-		g_Players[i].isAlly[g_ViewedPlayer] ? translate("Ally") :
-		g_Players[i].isNeutral[g_ViewedPlayer] ? translate("Neutral") : translate("Enemy");
+			g_Players[i].isAlly[g_ViewedPlayer] ?
+				translate("Ally") :
+				g_Players[i].isNeutral[g_ViewedPlayer] ? translate("Neutral") : translate("Enemy");
 }
 
 function diplomacyFormatStanceButtons(i, hidden)
 {
 	for (let stance of ["Ally", "Neutral", "Enemy"])
 	{
-		let button = Engine.GetGUIObjectByName("diplomacyPlayer"+stance+"["+(i-1)+"]");
+		let button = Engine.GetGUIObjectByName("diplomacyPlayer" + stance + "[" + (i - 1) + "]");
 		button.hidden = hidden;
 		if (hidden)
 			continue;
@@ -453,16 +495,16 @@ function diplomacyFormatTributeButtons(i, hidden)
 	let r = 0;
 	for (let resCode of resCodes)
 	{
-		let button = Engine.GetGUIObjectByName("diplomacyPlayer["+(i-1)+"]_tribute["+r+"]");
+		let button = Engine.GetGUIObjectByName("diplomacyPlayer[" + (i - 1) + "]_tribute[" + r + "]");
 		if (!button)
 		{
 			warn("Current GUI limits prevent displaying more than " + r + " tribute buttons!");
 			break;
 		}
 
-		Engine.GetGUIObjectByName("diplomacyPlayer["+(i-1)+"]_tribute["+r+"]_image").sprite = "stretched:session/icons/resources/"+resCode+".png";
+		Engine.GetGUIObjectByName("diplomacyPlayer[" + (i - 1) + "]_tribute[" + r + "]_image").sprite = "stretched:session/icons/resources/" + resCode + ".png";
 		button.hidden = hidden;
-		setPanelObjectPosition(button, r, r+1, 0);
+		setPanelObjectPosition(button, r, r + 1, 0);
 		++r;
 		if (hidden)
 			continue;
@@ -491,7 +533,7 @@ function diplomacyFormatTributeButtons(i, hidden)
 				// This is in a closure so that we have access to `player`, `amounts`, and `multiplier` without some
 				// evil global variable hackery.
 				g_FlushTributing = function() {
-					Engine.PostNetworkCommand({ "type": "tribute", "player": i, "amounts":  amounts });
+					Engine.PostNetworkCommand({ "type": "tribute", "player": i, "amounts": amounts });
 					multiplier = 1;
 					button.tooltip = formatTributeTooltip(i, resCode, 100);
 				};
@@ -505,7 +547,7 @@ function diplomacyFormatTributeButtons(i, hidden)
 
 function diplomacyFormatAttackRequestButton(i, hidden)
 {
-	let button = Engine.GetGUIObjectByName("diplomacyAttackRequest["+(i-1)+"]");
+	let button = Engine.GetGUIObjectByName("diplomacyAttackRequest[" + (i - 1) + "]");
 	button.hidden = hidden;
 	if (hidden)
 		return;
@@ -519,13 +561,14 @@ function diplomacyFormatAttackRequestButton(i, hidden)
 
 function diplomacyFormatSpyRequestButton(i, hidden)
 {
-	let button = Engine.GetGUIObjectByName("diplomacySpyRequest["+(i-1)+"]");
+	let button = Engine.GetGUIObjectByName("diplomacySpyRequest[" + (i - 1) + "]");
 	let template = GetTemplateData("special/spy");
 	button.hidden = hidden || !template || GetSimState().players[g_ViewedPlayer].disabledTemplates["special/spy"];
 	if (button.hidden)
 		return;
 
-	button.enabled = controlsPlayer(g_ViewedPlayer);
+	button.enabled = controlsPlayer(g_ViewedPlayer) &&
+	                 !(g_BribeButtonsWaiting[g_ViewedPlayer] && g_BribeButtonsWaiting[g_ViewedPlayer].indexOf(i) != -1);
 	let modifier = "";
 	let tooltips = [translate("Bribe a random unit from this player and share its vision during a limited period.")];
 	if (!button.enabled)
@@ -556,22 +599,36 @@ function diplomacyFormatSpyRequestButton(i, hidden)
 				"cost": modifiedTemplate.cost,
 				"player": g_ViewedPlayer
 			});
+			let costRatio = Engine.GetTemplate("special/spy").VisionSharing.FailureCostRatio;
+			if (costRatio > 0)
+			{
+				tooltips.push(translate("A failed bribe will cost you:"));
+				for (let res in modifiedTemplate.cost)
+					modifiedTemplate.cost[res] = Math.floor(costRatio * modifiedTemplate.cost[res]);
+				tooltips.push(getEntityCostTooltip(modifiedTemplate));
+			}
+
 			if (neededResources)
 			{
 				if (button.enabled)
-					modifier = resourcesToAlphaMask(neededResources) +":";
+					modifier = resourcesToAlphaMask(neededResources) + ":";
 				button.enabled = false;
 				tooltips.push(getNeededResourcesTooltip(neededResources));
 			}
 		}
 	}
-	let icon = Engine.GetGUIObjectByName("diplomacySpyRequestImage["+(i-1)+"]");
+	let icon = Engine.GetGUIObjectByName("diplomacySpyRequestImage[" + (i - 1) + "]");
 	icon.sprite = modifier + "stretched:session/icons/bribes.png";
 	button.tooltip = tooltips.filter(tip => tip).join("\n");
-	button.onPress = (function(i) { return function() {
+	button.onPress = (function(i, button) { return function() {
 		Engine.PostNetworkCommand({ "type": "spy-request", "source": g_ViewedPlayer, "player": i });
-		closeDiplomacy();
-	}; })(i);
+		if (!g_BribeButtonsWaiting[g_ViewedPlayer])
+			g_BribeButtonsWaiting[g_ViewedPlayer] = [];
+		// Don't push i twice
+		if (g_BribeButtonsWaiting[g_ViewedPlayer].indexOf(i) == -1)
+			g_BribeButtonsWaiting[g_ViewedPlayer].push(i);
+		diplomacyFormatSpyRequestButton(i, false);
+	}; })(i, button);
 }
 
 function resizeTradeDialog()
@@ -630,34 +687,34 @@ function openTrade()
 
 		// Barter:
 		barterOpenCommon(resCode, i, "barter");
-		setPanelObjectPosition(barterResource, i, i+1);
+		setPanelObjectPosition(barterResource, i, i + 1);
 
 		// Trade:
-		let tradeResource = Engine.GetGUIObjectByName("tradeResource["+i+"]");
+		let tradeResource = Engine.GetGUIObjectByName("tradeResource[" + i + "]");
 		if (!tradeResource)
 		{
 			 warn("Current GUI limits prevent displaying more than " + i + " resources in the trading goods selection dialog!");
 			 break;
 		}
 
-		setPanelObjectPosition(tradeResource, i, i+1);
+		setPanelObjectPosition(tradeResource, i, i + 1);
 
-		let icon = Engine.GetGUIObjectByName("tradeResourceIcon["+i+"]");
+		let icon = Engine.GetGUIObjectByName("tradeResourceIcon[" + i + "]");
 		icon.sprite = "stretched:session/icons/resources/" + resCode + ".png";
 
-		let buttonUp = Engine.GetGUIObjectByName("tradeArrowUp["+i+"]");
-		let buttonDn = Engine.GetGUIObjectByName("tradeArrowDn["+i+"]");
+		let buttonUp = Engine.GetGUIObjectByName("tradeArrowUp[" + i + "]");
+		let buttonDn = Engine.GetGUIObjectByName("tradeArrowDn[" + i + "]");
 
 		button[resCode] = {
 			"up": buttonUp,
 			"dn": buttonDn,
-			"label": Engine.GetGUIObjectByName("tradeResourceText["+i+"]"),
-			"sel": Engine.GetGUIObjectByName("tradeResourceSelection["+i+"]")
+			"label": Engine.GetGUIObjectByName("tradeResourceText[" + i + "]"),
+			"sel": Engine.GetGUIObjectByName("tradeResourceSelection[" + i + "]")
 		};
 
 		proba[resCode] = proba[resCode] || 0;
 
-		let buttonResource = Engine.GetGUIObjectByName("tradeResourceButton["+i+"]");
+		let buttonResource = Engine.GetGUIObjectByName("tradeResourceButton[" + i + "]");
 		buttonResource.enabled = controlsPlayer(g_ViewedPlayer);
 		buttonResource.onPress = (resource => {
 			return () => {
@@ -666,7 +723,7 @@ function openTrade()
 					for (let res of resCodes)
 						proba[res] = 0;
 					proba[resource] = 100;
-					Engine.PostNetworkCommand({"type": "set-trading-goods", "tradingGoods": proba});
+					Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
 				}
 				currTradeSelection = resource;
 				updateTradeButtons();
@@ -678,7 +735,7 @@ function openTrade()
 			return () => {
 				proba[resource] += Math.min(STEP, proba[currTradeSelection]);
 				proba[currTradeSelection] -= Math.min(STEP, proba[currTradeSelection]);
-				Engine.PostNetworkCommand({"type": "set-trading-goods", "tradingGoods": proba});
+				Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
 				updateTradeButtons();
 			};
 		})(resCode);
@@ -688,7 +745,7 @@ function openTrade()
 			return () => {
 				proba[currTradeSelection] += Math.min(STEP, proba[resource]);
 				proba[resource] -= Math.min(STEP, proba[resource]);
-				Engine.PostNetworkCommand({"type": "set-trading-goods", "tradingGoods": proba});
+				Engine.PostNetworkCommand({ "type": "set-trading-goods", "tradingGoods": proba });
 				updateTradeButtons();
 			};
 		})(resCode);
@@ -706,6 +763,11 @@ function updateTraderTexts()
 	Engine.GetGUIObjectByName("traderCountText").caption = getIdleLandTradersText(traderNumber) + "\n\n" + getIdleShipTradersText(traderNumber);
 }
 
+function initBarterButtons()
+{
+	g_BarterSell = g_ResourceData.GetCodes()[0];
+}
+
 /**
  * Code common to both the Barter Panel and the Trade/Barter Dialog, that
  * only needs to be run when the panel or dialog is opened by the player.
@@ -810,12 +872,8 @@ function updateBarterButtons()
 	Engine.GetGUIObjectByName("barterResources").hidden = !canBarter;
 	Engine.GetGUIObjectByName("barterHelp").hidden = !canBarter;
 
-	if (!canBarter)
-		return;
-
-	let resCodes = g_ResourceData.GetCodes();
-	for (let i = 0; i < resCodes.length; ++i)
-		barterUpdateCommon(resCodes[i], i, "barter", g_ViewedPlayer);
+	if (canBarter)
+		g_ResourceData.GetCodes().forEach((resCode, i) => { barterUpdateCommon(resCode, i, "barter", g_ViewedPlayer) });
 }
 
 function getIdleLandTradersText(traderNumber)
@@ -828,11 +886,11 @@ function getIdleLandTradersText(traderNumber)
 		"active": {
 			"garrisoned": {
 				"no-inactive": translate("%(openingTradingString)s, and %(garrisonedString)s."),
-				"inactive":    translate("%(openingTradingString)s, %(garrisonedString)s, and %(inactiveString)s.")
+				"inactive": translate("%(openingTradingString)s, %(garrisonedString)s, and %(inactiveString)s.")
 			},
 			"no-garrisoned": {
 				"no-inactive": translate("%(openingTradingString)s."),
-				"inactive":    translate("%(openingTradingString)s, and %(inactiveString)s.")
+				"inactive": translate("%(openingTradingString)s, and %(inactiveString)s.")
 			}
 		},
 		"no-active": {
@@ -858,31 +916,33 @@ function getIdleLandTradersText(traderNumber)
 		{ "numberTrading": active }
 	);
 
-	let inactiveString = sprintf(active || garrisoned ?
-		translatePlural(
-			"%(numberOfLandTraders)s inactive",
-			"%(numberOfLandTraders)s inactive",
-			inactive
-		) :
-		translatePlural(
-			"%(numberOfLandTraders)s land trader inactive",
-			"%(numberOfLandTraders)s land traders inactive",
-			inactive
-		),
+	let inactiveString = sprintf(
+		active || garrisoned ?
+			translatePlural(
+				"%(numberOfLandTraders)s inactive",
+				"%(numberOfLandTraders)s inactive",
+				inactive
+			) :
+			translatePlural(
+				"%(numberOfLandTraders)s land trader inactive",
+				"%(numberOfLandTraders)s land traders inactive",
+				inactive
+			),
 		{ "numberOfLandTraders": inactive }
 	);
 
-	let garrisonedString = sprintf(active || inactive ?
-		translatePlural(
-			"%(numberGarrisoned)s garrisoned on a trading merchant ship",
-			"%(numberGarrisoned)s garrisoned on a trading merchant ship",
-			garrisoned
-		) :
-		translatePlural(
-			"There is %(numberGarrisoned)s land trader garrisoned on a trading merchant ship",
-			"There are %(numberGarrisoned)s land traders garrisoned on a trading merchant ship",
-			garrisoned
-		),
+	let garrisonedString = sprintf(
+		active || inactive ?
+			translatePlural(
+				"%(numberGarrisoned)s garrisoned on a trading merchant ship",
+				"%(numberGarrisoned)s garrisoned on a trading merchant ship",
+				garrisoned
+			) :
+			translatePlural(
+				"There is %(numberGarrisoned)s land trader garrisoned on a trading merchant ship",
+				"There are %(numberGarrisoned)s land traders garrisoned on a trading merchant ship",
+				garrisoned
+			),
 		{ "numberGarrisoned": garrisoned }
 	);
 
@@ -890,7 +950,7 @@ function getIdleLandTradersText(traderNumber)
 		"openingTradingString": activeString,
 		"openingGarrisonedString": garrisonedString,
 		"garrisonedString": garrisonedString,
-		"inactiveString": "[color=\"" + g_IdleTraderTextColor + "\"]" + inactiveString + "[/color]"
+		"inactiveString": coloredText(inactiveString, g_IdleTraderTextColor)
 	});
 }
 
@@ -921,23 +981,24 @@ function getIdleShipTradersText(traderNumber)
 		{ "numberTrading": active }
 	);
 
-	let inactiveString = sprintf(active ?
-		translatePlural(
-			"%(numberOfShipTraders)s inactive",
-			"%(numberOfShipTraders)s inactive",
-			inactive
-		) :
-		translatePlural(
-			"%(numberOfShipTraders)s merchant ship inactive",
-			"%(numberOfShipTraders)s merchant ships inactive",
-			inactive
-		),
+	let inactiveString = sprintf(
+		active ?
+			translatePlural(
+				"%(numberOfShipTraders)s inactive",
+				"%(numberOfShipTraders)s inactive",
+				inactive
+			) :
+			translatePlural(
+				"%(numberOfShipTraders)s merchant ship inactive",
+				"%(numberOfShipTraders)s merchant ships inactive",
+				inactive
+			),
 		{ "numberOfShipTraders": inactive }
 	);
 
 	return sprintf(message, {
 		"openingTradingString": activeString,
-		"inactiveString": "[color=\"" + g_IdleTraderTextColor + "\"]" + inactiveString + "[/color]"
+		"inactiveString": coloredText(inactiveString, g_IdleTraderTextColor)
 	});
 }
 
@@ -956,6 +1017,36 @@ function toggleTrade()
 		openTrade();
 }
 
+function toggleTutorial()
+{
+	let tutorialPanel = Engine.GetGUIObjectByName("tutorialPanel");
+	tutorialPanel.hidden = !tutorialPanel.hidden ||
+	                       !Engine.GetGUIObjectByName("tutorialText").caption;
+}
+
+function updateGameSpeedControl()
+{
+	Engine.GetGUIObjectByName("gameSpeedButton").hidden = g_IsNetworked;
+
+	let player = g_Players[Engine.GetPlayerID()];
+	g_GameSpeeds = getGameSpeedChoices(!player || player.state != "active");
+
+	let gameSpeed = Engine.GetGUIObjectByName("gameSpeed");
+	gameSpeed.list = g_GameSpeeds.Title;
+	gameSpeed.list_data = g_GameSpeeds.Speed;
+
+	let simRate = Engine.GetSimRate();
+
+	let gameSpeedIdx = g_GameSpeeds.Speed.indexOf(+simRate.toFixed(2));
+	if (gameSpeedIdx == -1)
+		warn("Unknown gamespeed:" + simRate);
+
+	gameSpeed.selected = gameSpeedIdx != -1 ? gameSpeedIdx : g_GameSpeeds.Default;
+	gameSpeed.onSelectionChange = function() {
+		changeGameSpeed(+this.list_data[this.selected]);
+	};
+}
+
 function toggleGameSpeed()
 {
 	let gameSpeed = Engine.GetGUIObjectByName("gameSpeed");
@@ -979,7 +1070,7 @@ function openObjectives()
 	let playerState = player && player.state;
 	let isActive = !playerState || playerState == "active";
 
-	Engine.GetGUIObjectByName("gameDescriptionText").caption = getGameDescription(true);
+	Engine.GetGUIObjectByName("gameDescriptionText").caption = getGameDescription();
 
 	let objectivesPlayerstate = Engine.GetGUIObjectByName("objectivesPlayerstate");
 	objectivesPlayerstate.hidden = isActive;
@@ -1013,15 +1104,17 @@ function openGameSummary()
 	Engine.PushGuiPage("page_summary.xml", {
 		"sim": {
 			"mapSettings": g_GameAttributes.settings,
-			"playerStates":extendedSimState.players.filter((state,player) =>
+			"playerStates": extendedSimState.players.filter((state, player) =>
 				g_IsObserver || player == 0 || player == g_ViewedPlayer ||
 				extendedSimState.players[g_ViewedPlayer].hasSharedLos && g_Players[player].isMutualAlly[g_ViewedPlayer]),
-			"timeElapsed" : extendedSimState.timeElapsed
+			"timeElapsed": extendedSimState.timeElapsed
 		},
 		"gui": {
+			"dialog": true,
 			"isInGame": true
 		},
-		"callback": "resumeGame"
+		"selectedData": g_SummarySelectedData,
+		"callback": "resumeGameAndSaveSummarySelectedData"
 	});
 }
 
@@ -1031,12 +1124,20 @@ function openStrucTree()
 	pauseGame();
 
 	// TODO add info about researched techs and unlocked entities
-	Engine.PushGuiPage("page_structree.xml", {
-		"civ" : g_Players[g_ViewedPlayer].civ,
-		"callback": "resumeGame",
+
+	Engine.PushGuiPage(g_CivInfo.page, {
+		"civ": g_CivInfo.code || g_Players[g_ViewedPlayer].civ,
+		"callback": "storeCivInfoPage"
 	});
 }
 
+function storeCivInfoPage(data)
+{
+	g_CivInfo.code = data.civ;
+	g_CivInfo.page = data.page;
+	resumeGame();
+}
+
 /**
  * Pause or resume the game.
  *
@@ -1069,6 +1170,12 @@ function resumeGame(explicit = false)
 	pauseGame(false, explicit);
 }
 
+function resumeGameAndSaveSummarySelectedData(data)
+{
+	g_SummarySelectedData = data.summarySelectedData;
+	resumeGame(data.explicitResume);
+}
+
 /**
  * Called when the current player toggles a pause button.
  */
@@ -1140,18 +1247,21 @@ function toggleDeveloperOverlay()
 		markForTranslation("The Developer Overlay was closed.") :
 		markForTranslation("The Developer Overlay was opened.");
 
-	Engine.PostNetworkCommand({
-		"type": "aichat",
-		"message": message,
-		"translateMessage": true,
-		"translateParameters": [],
-		"parameters": {}
-	});
+	// Only players can send the simulation chat command
+	if (Engine.GetPlayerID() == -1)
+		submitChatDirectly(message);
+	else
+		Engine.PostNetworkCommand({
+			"type": "aichat",
+			"message": message,
+			"translateMessage": true,
+			"translateParameters": [],
+			"parameters": {}
+		});
 }
 
 function closeOpenDialogs()
 {
-	 // TODO: also close message boxes
 	closeMenu();
 	closeChat();
 	closeDiplomacy();
