@@ -1,13 +1,27 @@
 const g_IsReplay = Engine.IsVisualReplay();
 
+const g_CivData = loadCivData(false, true);
+
 const g_Ceasefire = prepareForDropdown(g_Settings && g_Settings.Ceasefire);
-const g_GameSpeeds = prepareForDropdown(g_Settings && g_Settings.GameSpeeds.filter(speed => !speed.ReplayOnly || g_IsReplay));
 const g_MapSizes = prepareForDropdown(g_Settings && g_Settings.MapSizes);
 const g_MapTypes = prepareForDropdown(g_Settings && g_Settings.MapTypes);
 const g_PopulationCapacities = prepareForDropdown(g_Settings && g_Settings.PopulationCapacities);
 const g_StartingResources = prepareForDropdown(g_Settings && g_Settings.StartingResources);
-const g_VictoryConditions = prepareForDropdown(g_Settings && g_Settings.VictoryConditions);
 const g_VictoryDurations = prepareForDropdown(g_Settings && g_Settings.VictoryDurations);
+const g_VictoryConditions = g_Settings && g_Settings.VictoryConditions;
+
+var g_GameSpeeds;
+
+/**
+ * Whether to display diplomacy colors (where players see self/ally/neutral/enemy each in different colors and
+ * observers see each team in a different color) or regular player colors.
+ */
+var g_DiplomacyColorsToggle = false;
+
+/**
+ * The array of displayed player colors (either the diplomacy color or regular color for each player).
+ */
+var g_DisplayedPlayerColors;
 
 /**
  * Colors to flash when pop limit reached.
@@ -23,22 +37,22 @@ var g_ResourceTitleFont = "sans-bold-16";
 /**
  * A random file will be played. TODO: more variety
  */
-const g_Ambient = [ "audio/ambient/dayscape/day_temperate_gen_03.ogg" ];
+var g_Ambient = ["audio/ambient/dayscape/day_temperate_gen_03.ogg"];
 
 /**
  * Map, player and match settings set in gamesetup.
  */
-const g_GameAttributes = Object.freeze(Engine.GetInitAttributes());
-
-/**
- * Is this user in control of game settings (i.e. is a network server, or offline player).
- */
-var g_IsController;
+const g_GameAttributes = deepfreeze(Engine.GetInitAttributes());
 
 /**
  * True if this is a multiplayer game.
  */
-var g_IsNetworked = false;
+const g_IsNetworked = Engine.HasNetClient();
+
+/**
+ * Is this user in control of game settings (i.e. is a network server, or offline player).
+ */
+var g_IsController = !g_IsNetworked || Engine.HasNetServer();
 
 /**
  * Whether we have finished the synchronization and
@@ -101,24 +115,18 @@ var g_LastTickTime = Date.now();
 /**
  * Recalculate which units have their status bars shown with this frequency in milliseconds.
  */
-const g_StatusBarUpdate = 200;
-
-/**
- * Not constant as we add "gaia".
- */
-var g_CivData = {};
+var g_StatusBarUpdate = 200;
 
 /**
  * For restoring selection, order and filters when returning to the replay menu
  */
 var g_ReplaySelectionData;
 
-var g_PlayerAssignments = {
-	"local": {
-		"name": singleplayerName(),
-		"player": 1
-	}
-};
+/**
+ * Remembers which clients are assigned to which player slots.
+ * The keys are guids or "local" in Singleplayer.
+ */
+var g_PlayerAssignments;
 
 /**
  * Cache dev-mode settings that are frequently or widely used.
@@ -145,13 +153,13 @@ var g_ShowAllStatusBars = false;
 var g_IsTrainingBlocked = false;
 
 /**
- * Cache simulation state (updated on every simulation update).
+ * Cache of simulation state and template data (apart from TechnologyData, updated on every simulation update).
  */
 var g_SimState;
 var g_EntityStates = {};
 var g_TemplateData = {};
-var g_TemplateDataWithoutLocalization = {};
 var g_TechnologyData = {};
+
 var g_ResourceData = new Resources();
 
 /**
@@ -186,64 +194,48 @@ var g_WorkerTypes = ["FemaleCitizen", "Trader", "FishingBoat", "CitizenSoldier"]
  * Unit classes to be checked for the military-only-selection modifier and for the idle-warrior-hotkey.
  */
 var g_MilitaryTypes = ["Melee", "Ranged"];
-/**
- * Cache the idle worker status.
- */
-var g_HasIdleWorker = false;
 
 function GetSimState()
 {
 	if (!g_SimState)
-		g_SimState = Engine.GuiInterfaceCall("GetSimulationState");
+		g_SimState = deepfreeze(Engine.GuiInterfaceCall("GetSimulationState"));
 
 	return g_SimState;
 }
 
+function GetMultipleEntityStates(ents)
+{
+	if (!ents.length)
+		return null;
+	let entityStates = Engine.GuiInterfaceCall("GetMultipleEntityStates", ents);
+	for (let item of entityStates)
+		g_EntityStates[item.entId] = item.state && deepfreeze(item.state);
+	return entityStates;
+}
+
 function GetEntityState(entId)
 {
 	if (!g_EntityStates[entId])
-		g_EntityStates[entId] = Engine.GuiInterfaceCall("GetEntityState", entId);
+	{
+		let entityState = Engine.GuiInterfaceCall("GetEntityState", entId);
+		g_EntityStates[entId] = entityState && deepfreeze(entityState);
+	}
 
 	return g_EntityStates[entId];
 }
 
-function GetExtendedEntityState(entId)
-{
-	let entState = GetEntityState(entId);
-	if (!entState || entState.extended)
-		return entState;
-
-	let extension = Engine.GuiInterfaceCall("GetExtendedEntityState", entId);
-	for (let prop in extension)
-		entState[prop] = extension[prop];
-	entState.extended = true;
-	g_EntityStates[entId] = entState;
-	return entState;
-}
-
 function GetTemplateData(templateName)
 {
 	if (!(templateName in g_TemplateData))
 	{
 		let template = Engine.GuiInterfaceCall("GetTemplateData", templateName);
 		translateObjectKeys(template, ["specific", "generic", "tooltip"]);
-		g_TemplateData[templateName] = template;
+		g_TemplateData[templateName] = deepfreeze(template);
 	}
 
 	return g_TemplateData[templateName];
 }
 
-function GetTemplateDataWithoutLocalization(templateName)
-{
-	if (!(templateName in g_TemplateDataWithoutLocalization))
-	{
-		let template = Engine.GuiInterfaceCall("GetTemplateData", templateName);
-		g_TemplateDataWithoutLocalization[templateName] = template;
-	}
-
-	return g_TemplateDataWithoutLocalization[templateName];
-}
-
 function GetTechnologyData(technologyName, civ)
 {
 	if (!g_TechnologyData[civ])
@@ -251,9 +243,9 @@ function GetTechnologyData(technologyName, civ)
 
 	if (!(technologyName in g_TechnologyData[civ]))
 	{
-		let template = Engine.GuiInterfaceCall("GetTechnologyData", { "name": technologyName, "civ": civ });
+		let template = GetTechnologyDataHelper(TechnologyTemplates.Get(technologyName), civ, g_ResourceData);
 		translateObjectKeys(template, ["specific", "generic", "description", "tooltip", "requirementsTooltip"]);
-		g_TechnologyData[civ][technologyName] = template;
+		g_TechnologyData[civ][technologyName] = deepfreeze(template);
 	}
 
 	return g_TechnologyData[civ][technologyName];
@@ -268,73 +260,30 @@ function init(initData, hotloadData)
 		return;
 	}
 
+	// Fallback used by atlas
+	g_PlayerAssignments = initData ? initData.playerAssignments : { "local": { "player": 1 } };
+
+	// Fallback used by atlas and autostart games
+	if (g_PlayerAssignments.local && !g_PlayerAssignments.local.name)
+		g_PlayerAssignments.local.name = singleplayerName();
+
 	if (initData)
 	{
-		g_IsNetworked = initData.isNetworked;
-		g_IsController = initData.isController;
-		g_PlayerAssignments = initData.playerAssignments;
 		g_ReplaySelectionData = initData.replaySelectionData;
 		g_HasRejoined = initData.isRejoining;
 
 		if (initData.savedGUIData)
 			restoreSavedGameData(initData.savedGUIData);
-
-		Engine.GetGUIObjectByName("gameSpeedButton").hidden = g_IsNetworked;
-	}
-	else // Needed for autostart loading option
-	{
-		if (g_IsReplay)
-			g_PlayerAssignments.local.player = -1;
 	}
 
+	LoadModificationTemplates();
 	updatePlayerData();
-
-	g_CivData = loadCivData();
-	g_CivData.gaia = { "Code": "gaia", "Name": translate("Gaia") };
-
-	g_BarterSell = g_ResourceData.GetCodes()[0];
-
 	initializeMusic(); // before changing the perspective
-
-	let gameSpeed = Engine.GetGUIObjectByName("gameSpeed");
-	gameSpeed.list = g_GameSpeeds.Title;
-	gameSpeed.list_data = g_GameSpeeds.Speed;
-	let gameSpeedIdx = g_GameSpeeds.Speed.indexOf(Engine.GetSimRate());
-	gameSpeed.selected = gameSpeedIdx != -1 ? gameSpeedIdx : g_GameSpeeds.Default;
-	gameSpeed.onSelectionChange = function() { changeGameSpeed(+this.list_data[this.selected]); };
-
-	initMenuPosition();
-	resizeDiplomacyDialog();
-	resizeTradeDialog();
-
-	for (let slot in Engine.GetGUIObjectByName("panelEntityPanel").children)
-		initPanelEntities(slot);
-
-	// Populate player selection dropdown
-	let playerNames = [translate("Observer")];
-	let playerIDs = [-1];
-	for (let player in g_Players)
-	{
-		playerIDs.push(player);
-		playerNames.push(colorizePlayernameHelper("■", player) + " " + g_Players[player].name);
-	}
-
-	// Select "observer" item when rejoining as a defeated player
-	let viewedPlayer = g_Players[Engine.GetPlayerID()];
-	let viewPlayerDropdown = Engine.GetGUIObjectByName("viewPlayer");
-	viewPlayerDropdown.list = playerNames;
-	viewPlayerDropdown.list_data = playerIDs;
-	viewPlayerDropdown.selected = viewedPlayer && viewedPlayer.state == "defeated" ? 0 : Engine.GetPlayerID() + 1;
-
-	// If in Atlas editor, disable the exit button
-	if (Engine.IsAtlasRunning())
-		Engine.GetGUIObjectByName("menuExitButton").enabled = false;
+	initGUIObjects();
 
 	if (hotloadData)
 		g_Selection.selected = hotloadData.selection;
 
-	initChatWindow();
-
 	sendLobbyPlayerlistUpdate();
 	onSimulationUpdate();
 	setTimeout(displayGamestateNotifications, 1000);
@@ -346,8 +295,22 @@ function init(initData, hotloadData)
 	//
 	// DISABLED: this information isn't currently useful for anything much,
 	// and it generates a massive amount of data to transmit and store
-	//setTimeout(function() { reportPerformance(5); }, 5000);
-	//setTimeout(function() { reportPerformance(60); }, 60000);
+	// setTimeout(function() { reportPerformance(5); }, 5000);
+	// setTimeout(function() { reportPerformance(60); }, 60000);
+}
+
+function initGUIObjects()
+{
+	initMenu();
+	updateGameSpeedControl();
+	resizeDiplomacyDialog();
+	resizeTradeDialog();
+	initBarterButtons();
+	initPanelEntities();
+	initViewedPlayerDropdown();
+	initChatWindow();
+	Engine.SetBoundingBoxDebugOverlay(false);
+	updateEnabledRangeOverlayTypes();
 }
 
 function updatePlayerData()
@@ -398,6 +361,71 @@ function updatePlayerData()
 	g_Players = playerData;
 }
 
+function updateDiplomacyColorsButton()
+{
+	g_DiplomacyColorsToggle = !g_DiplomacyColorsToggle;
+
+	let diplomacyColorsButton = Engine.GetGUIObjectByName("diplomacyColorsButton");
+
+	diplomacyColorsButton.sprite = g_DiplomacyColorsToggle ?
+		"stretched:session/minimap-diplomacy-on.png" :
+		"stretched:session/minimap-diplomacy-off.png";
+
+	diplomacyColorsButton.sprite_over = g_DiplomacyColorsToggle ?
+		"stretched:session/minimap-diplomacy-on-highlight.png" :
+		"stretched:session/minimap-diplomacy-off-highlight.png";
+
+	Engine.GetGUIObjectByName("diplomacyColorsWindowButtonIcon").sprite = g_DiplomacyColorsToggle ?
+		"stretched:session/icons/diplomacy-on.png" :
+		"stretched:session/icons/diplomacy.png";
+
+	updateDisplayedPlayerColors();
+}
+
+/**
+ * Updates the displayed colors of players in the simulation and GUI.
+ */
+function updateDisplayedPlayerColors()
+{
+	if (g_DiplomacyColorsToggle)
+	{
+		let getDiplomacyColor = stance =>
+			guiToRgbColor(Engine.ConfigDB_GetValue("user", "gui.session.diplomacycolors." + stance)) ||
+			guiToRgbColor(Engine.ConfigDB_GetValue("default", "gui.session.diplomacycolors." + stance));
+
+		let teamRepresentatives = {};
+		for (let i = 1; i < g_Players.length; ++i)
+			if (g_ViewedPlayer <= 0)
+			{
+				// Observers and gaia see team colors
+				let team = g_Players[i].team;
+				g_DisplayedPlayerColors[i] = g_Players[teamRepresentatives[team] || i].color;
+				if (team != -1 && !teamRepresentatives[team])
+					teamRepresentatives[team] = i;
+			}
+			else
+				// Players see colors depending on diplomacy
+				g_DisplayedPlayerColors[i] =
+					g_ViewedPlayer == i ? getDiplomacyColor("self") :
+					g_Players[g_ViewedPlayer].isAlly[i] ? getDiplomacyColor("ally") :
+					g_Players[g_ViewedPlayer].isNeutral[i] ? getDiplomacyColor("neutral") :
+					getDiplomacyColor("enemy");
+
+		g_DisplayedPlayerColors[0] = g_Players[0].color;
+	}
+	else
+		g_DisplayedPlayerColors = g_Players.map(player => player.color);
+
+	Engine.GuiInterfaceCall("UpdateDisplayedPlayerColors", {
+		"displayedPlayerColors": g_DisplayedPlayerColors,
+		"displayDiplomacyColors": g_DiplomacyColorsToggle,
+		"showAllStatusBars": g_ShowAllStatusBars,
+		"selected": g_Selection.toList()
+	});
+
+	updateGUIObjects();
+}
+
 /**
  * Depends on the current player (g_IsObserver).
  */
@@ -418,6 +446,14 @@ function updateHotkeyTooltips()
 		colorizeHotkey("%(hotkey)s" + " ", "selection.idleworker") +
 		translate("Find idle worker");
 
+	Engine.GetGUIObjectByName("diplomacyColorsButton").tooltip =
+		colorizeHotkey("%(hotkey)s" + " ", "session.diplomacycolors") +
+		translate("Toggle Diplomacy Colors");
+
+	Engine.GetGUIObjectByName("diplomacyColorsWindowButton").tooltip =
+		colorizeHotkey("%(hotkey)s" + " ", "session.diplomacycolors") +
+		translate("Toggle Diplomacy Colors");
+
 	Engine.GetGUIObjectByName("tradeHelp").tooltip = colorizeHotkey(
 		translate("Select one type of goods you want to modify by clicking on it, and then use the arrows of the other types to modify their shares. You can also press %(hotkey)s while selecting one type of goods to bring its share to 100%%."),
 		"session.fulltradeswap");
@@ -430,26 +466,27 @@ function updateHotkeyTooltips()
 		});
 }
 
-function initPanelEntities(slot)
+function initPanelEntities()
 {
-	let button = Engine.GetGUIObjectByName("panelEntityButton[" + slot + "]");
+	Engine.GetGUIObjectByName("panelEntityPanel").children.forEach((button, slot) => {
 
-	button.onPress = function() {
-		let panelEnt = g_PanelEntities.find(panelEnt => panelEnt.slot !== undefined && panelEnt.slot == slot);
-		if (!panelEnt)
-			return;
+		button.onPress = function() {
+			let panelEnt = g_PanelEntities.find(ent => ent.slot !== undefined && ent.slot == slot);
+			if (!panelEnt)
+				return;
 
-		if (!Engine.HotkeyIsPressed("selection.add"))
-			g_Selection.reset();
+			if (!Engine.HotkeyIsPressed("selection.add"))
+				g_Selection.reset();
 
-		g_Selection.addList([panelEnt.ent]);
-	};
+			g_Selection.addList([panelEnt.ent]);
+		};
 
-	button.onDoublePress = function() {
-		let panelEnt = g_PanelEntities.find(panelEnt => panelEnt.slot !== undefined && panelEnt.slot == slot);
-		if (panelEnt)
-			selectAndMoveTo(getEntityOrHolder(panelEnt.ent));
-	};
+		button.onDoublePress = function() {
+			let panelEnt = g_PanelEntities.find(ent => ent.slot !== undefined && ent.slot == slot);
+			if (panelEnt)
+				selectAndMoveTo(getEntityOrHolder(panelEnt.ent));
+		};
+	});
 }
 
 /**
@@ -459,7 +496,7 @@ function getEntityOrHolder(ent)
 {
 	let entState = GetEntityState(ent);
 	if (entState && !entState.position && entState.unitAI && entState.unitAI.orders.length &&
-			(entState.unitAI.orders[0].type == "Garrison" || entState.unitAI.orders[0].type == "Autogarrison"))
+			entState.unitAI.orders[0].type == "Garrison")
 		return getEntityOrHolder(entState.unitAI.orders[0].data.target);
 
 	return ent;
@@ -474,6 +511,25 @@ function initializeMusic()
 	playAmbient();
 }
 
+function initViewedPlayerDropdown()
+{
+	g_DisplayedPlayerColors = g_Players.map(player => player.color);
+	updateViewedPlayerDropdown();
+
+	// Select "observer" in the view player dropdown when rejoining as a defeated player
+	let player = g_Players[Engine.GetPlayerID()];
+	Engine.GetGUIObjectByName("viewPlayer").selected = player && player.state == "defeated" ? 0 : Engine.GetPlayerID() + 1;
+}
+
+function updateViewedPlayerDropdown()
+{
+	let viewPlayer = Engine.GetGUIObjectByName("viewPlayer");
+	viewPlayer.list_data = [-1].concat(g_Players.map((player, i) => i));
+	viewPlayer.list = [translate("Observer")].concat(g_Players.map(
+		(player, i) => colorizePlayernameHelper("■", i) + " " + player.name
+	));
+}
+
 function toggleChangePerspective(enabled)
 {
 	g_DevSettings.changePerspective = enabled;
@@ -508,11 +564,14 @@ function selectViewPlayer(playerID)
 	}
 
 	Engine.SetViewedPlayer(g_ViewedPlayer);
+	updateDisplayedPlayerColors();
 	updateTopPanel();
 	updateChatAddressees();
 	updateHotkeyTooltips();
 
 	// Update GUI and clear player-dependent cache
+	g_TemplateData = {};
+	Engine.GuiInterfaceCall("ResetTemplateModified");
 	onSimulationUpdate();
 
 	if (g_IsDiplomacyOpen)
@@ -546,23 +605,36 @@ function controlsPlayer(playerID)
 }
 
 /**
- * Called when a player has won or was defeated.
+ * Called when one or more players have won or were defeated.
+ *
+ * @param {array} - IDs of the players who have won or were defeated.
+ * @param {object} - a plural string stating the victory reason.
+ * @param {boolean} - whether these players have won or lost.
  */
-function playerFinished(player, won)
+function playersFinished(players, victoryString, won)
 {
-	if (player == Engine.GetPlayerID())
+	addChatMessage({
+		"type": "defeat-victory",
+		"message": victoryString,
+		"players": players
+	});
+
+	if (players.indexOf(Engine.GetPlayerID()) != -1)
 		reportGame();
 
+	sendLobbyPlayerlistUpdate();
+
 	updatePlayerData();
 	updateChatAddressees();
+	updateGameSpeedControl();
 
-	if (player != g_ViewedPlayer)
+	if (players.indexOf(g_ViewedPlayer) == -1)
 		return;
 
 	// Select "observer" item on loss. On win enable observermode without changing perspective
 	Engine.GetGUIObjectByName("viewPlayer").selected = won ? g_ViewedPlayer + 1 : 0;
 
-	if (player != Engine.GetPlayerID() || Engine.IsAtlasRunning())
+	if (players.indexOf(Engine.GetPlayerID()) == -1 || Engine.IsAtlasRunning())
 		return;
 
 	global.music.setState(
@@ -587,12 +659,17 @@ function updateTopPanel()
 	if (isPlayer)
 	{
 		civIcon.sprite = "stretched:" + g_CivData[g_Players[g_ViewedPlayer].civ].Emblem;
-		Engine.GetGUIObjectByName("civIconOverlay").tooltip = sprintf(translate("%(civ)s - Structure Tree"), {
-			"civ": g_CivData[g_Players[g_ViewedPlayer].civ].Name
-		});
+		Engine.GetGUIObjectByName("civIconOverlay").tooltip =
+			sprintf(
+				translate("%(civ)s\n%(hotkey_civinfo)s / %(hotkey_structree)s: View History / Structure Tree\nLast opened will be reopened on click."), {
+					"civ": setStringTags(g_CivData[g_Players[g_ViewedPlayer].civ].Name, { "font": "sans-bold-stroke-14" }),
+					"hotkey_civinfo": colorizeHotkey("%(hotkey)s", "civinfo"),
+					"hotkey_structree": colorizeHotkey("%(hotkey)s", "structree")
+			});
 	}
 
-	Engine.GetGUIObjectByName("optionFollowPlayer").hidden = !g_IsObserver || !isPlayer;
+	// Following gaia can be interesting on scripted maps
+	Engine.GetGUIObjectByName("optionFollowPlayer").hidden = !g_IsObserver || g_ViewedPlayer == -1;
 
 	let viewPlayer = Engine.GetGUIObjectByName("viewPlayer");
 	viewPlayer.hidden = !g_IsObserver && !g_DevSettings.changePerspective;
@@ -605,13 +682,13 @@ function updateTopPanel()
 	let r = 0;
 	for (let res of resCodes)
 	{
-		if (!Engine.GetGUIObjectByName("resource["+r+"]"))
+		if (!Engine.GetGUIObjectByName("resource[" + r + "]"))
 		{
 			warn("Current GUI limits prevent displaying more than " + r + " resources in the top panel!");
 			break;
 		}
-		Engine.GetGUIObjectByName("resource["+r+"]_icon").sprite = "stretched:session/icons/resources/" + res + ".png";
-		Engine.GetGUIObjectByName("resource["+r+"]").hidden = !isPlayer;
+		Engine.GetGUIObjectByName("resource[" + r + "]_icon").sprite = "stretched:session/icons/resources/" + res + ".png";
+		Engine.GetGUIObjectByName("resource[" + r + "]").hidden = !isPlayer;
 		++r;
 	}
 	horizontallySpaceObjects("resourceCounts", 5);
@@ -619,20 +696,21 @@ function updateTopPanel()
 
 	let resPop = Engine.GetGUIObjectByName("population");
 	let resPopSize = resPop.size;
-	resPopSize.left = Engine.GetGUIObjectByName("resource["+ (r-1) +"]").size.right;
+	resPopSize.left = Engine.GetGUIObjectByName("resource[" + (r - 1) + "]").size.right;
 	resPop.size = resPopSize;
 
 	Engine.GetGUIObjectByName("population").hidden = !isPlayer;
-	Engine.GetGUIObjectByName("diplomacyButton1").hidden = !isPlayer;
-	Engine.GetGUIObjectByName("tradeButton1").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("diplomacyButton").hidden = !isPlayer;
+	Engine.GetGUIObjectByName("tradeButton").hidden = !isPlayer;
 	Engine.GetGUIObjectByName("observerText").hidden = isPlayer;
 
 	let alphaLabel = Engine.GetGUIObjectByName("alphaLabel");
 	alphaLabel.hidden = isPlayer && !viewPlayer.hidden;
-	alphaLabel.size = isPlayer ? "50%+54 0 100%-293 100%" : "155 0 85%-279 100%";
+	alphaLabel.size = isPlayer ? "50%+44 0 100%-283 100%" : "155 0 85%-279 100%";
 
 	Engine.GetGUIObjectByName("pauseButton").enabled = !g_IsObserver || !g_IsNetworked || g_IsController;
 	Engine.GetGUIObjectByName("menuResignButton").enabled = !g_IsObserver;
+	Engine.GetGUIObjectByName("lobbyButton").enabled = Engine.HasXmppClient();
 }
 
 function reportPerformance(time)
@@ -657,9 +735,7 @@ function resignGame(leaveGameAfterResign)
 		return;
 
 	Engine.PostNetworkCommand({
-		"type": "defeat-player",
-		"playerId": Engine.GetPlayerID(),
-		"resign": true
+		"type": "resign"
 	});
 
 	if (!leaveGameAfterResign)
@@ -678,6 +754,7 @@ function leaveGame(willRejoin)
 	// Before ending the game
 	let replayDirectory = Engine.GetCurrentReplayDirectory();
 	let simData = getReplayMetadata();
+	let playerID = Engine.GetPlayerID();
 
 	Engine.EndGame();
 
@@ -692,7 +769,8 @@ function leaveGame(willRejoin)
 	Engine.SwitchGuiPage("page_summary.xml", {
 		"sim": simData,
 		"gui": {
-			"assignedPlayer": Engine.GetPlayerID(),
+			"dialog": false,
+			"assignedPlayer": playerID,
 			"disconnected": g_Disconnected,
 			"isReplay": g_IsReplay,
 			"replayDirectory": !g_HasRejoined && replayDirectory,
@@ -752,6 +830,8 @@ function onTick()
 	if (g_Selection.dirty)
 	{
 		g_Selection.dirty = false;
+		// When selection changed, get the entityStates of new entities
+		GetMultipleEntityStates(g_Selection.toList().filter(entId => !g_EntityStates[entId]));
 
 		updateGUIObjects();
 
@@ -786,49 +866,36 @@ function changeGameSpeed(speed)
 		Engine.SetSimRate(speed);
 }
 
-function hasIdleWorker()
-{
-	return Engine.GuiInterfaceCall("HasIdleUnits", {
-			"viewedPlayer": g_ViewedPlayer,
-			"idleClasses": g_WorkerTypes,
-			"excludeUnits": []
-	});
-}
-
 function updateIdleWorkerButton()
 {
-	g_HasIdleWorker = hasIdleWorker();
-
-	let idleWorkerButton = Engine.GetGUIObjectByName("idleOverlay");
-	let prefix = "stretched:session/";
-
-	if (!g_HasIdleWorker)
-		idleWorkerButton.sprite = prefix + "minimap-idle-disabled.png";
-	else if (idleWorkerButton.sprite != prefix + "minimap-idle-highlight.png")
-		idleWorkerButton.sprite = prefix + "minimap-idle.png";
+	Engine.GetGUIObjectByName("idleWorkerButton").enabled = Engine.GuiInterfaceCall("HasIdleUnits", {
+		"viewedPlayer": g_ViewedPlayer,
+		"idleClasses": g_WorkerTypes,
+		"excludeUnits": []
+	});
 }
 
 function onSimulationUpdate()
 {
+	// Templates change depending on technologies and auras, so they have to be reloaded after such a change.
+	// g_TechnologyData data never changes, so it shouldn't be deleted.
 	g_EntityStates = {};
-	g_TemplateData = {};
-	g_TechnologyData = {};
+	if (Engine.GuiInterfaceCall("IsTemplateModified"))
+	{
+		g_TemplateData = {};
+		Engine.GuiInterfaceCall("ResetTemplateModified");
+	}
+	g_SimState = undefined;
 
-	g_SimState = Engine.GuiInterfaceCall("GetSimulationState");
-
-	if (!g_SimState)
+	if (!GetSimState())
 		return;
 
+	GetMultipleEntityStates(g_Selection.toList());
+
 	updateCinemaPath();
 	handleNotifications();
 	updateGUIObjects();
 
-	for (let type of ["Aura", "Heal"])
-		Engine.GuiInterfaceCall("EnableVisualRangeOverlayType", {
-			"type": type,
-			"enabled": Engine.ConfigDB_GetValue("user", "gui.session." + type.toLowerCase() + "range") == "true"
-		});
-
 	if (g_ConfirmExit)
 		confirmExit();
 }
@@ -844,8 +911,7 @@ function confirmExit()
 	closeOpenDialogs();
 
 	// Don't ask for exit if other humans are still playing
-	let isHost = g_IsController && g_IsNetworked;
-	let askExit = !isHost || isHost && g_Players.every((player, i) =>
+	let askExit = !Engine.HasNetServer() || g_Players.every((player, i) =>
 		i == 0 ||
 		player.state != "active" ||
 		g_GameAttributes.settings.PlayerData[i].AI != "");
@@ -867,11 +933,17 @@ function confirmExit()
 	g_ConfirmExit = false;
 }
 
+function toggleGUI()
+{
+	g_ShowGUI = !g_ShowGUI;
+	updateCinemaPath();
+}
+
 function updateCinemaPath()
 {
 	let isPlayingCinemaPath = GetSimState().cinemaPlaying && !g_Disconnected;
 
-	Engine.GetGUIObjectByName("sn").hidden = !g_ShowGUI || isPlayingCinemaPath;
+	Engine.GetGUIObjectByName("session").hidden = !g_ShowGUI || isPlayingCinemaPath;
 	Engine.Renderer_SetSilhouettesEnabled(!isPlayingCinemaPath && Engine.ConfigDB_GetValue("user", "silhouettes") == "true");
 }
 
@@ -918,6 +990,7 @@ function updateGUIObjects()
 			global.music.setState(global.music.states[battleState]);
 	}
 
+	updateViewedPlayerDropdown();
 	updateDiplomacy();
 }
 
@@ -952,7 +1025,7 @@ function updateGUIStatusBar(nameOfBar, points, maxPoints, direction)
 		return;
 
 	let healthSize = statusBar.size;
-	let value = 100*Math.max(0, Math.min(1, points / maxPoints));
+	let value = 100 * Math.max(0, Math.min(1, points / maxPoints));
 
 	// inverse bar
 	if (direction == 2 || direction == 3)
@@ -972,17 +1045,19 @@ function updateGUIStatusBar(nameOfBar, points, maxPoints, direction)
 
 function updatePanelEntities()
 {
-	let playerState = GetSimState().players[g_ViewedPlayer];
-	let panelEnts = playerState ? playerState.panelEntities : [];
+	let panelEnts =
+		g_ViewedPlayer == -1 ?
+			GetSimState().players.reduce((ents, pState) => ents.concat(pState.panelEntities), []) :
+			GetSimState().players[g_ViewedPlayer].panelEntities;
 
 	g_PanelEntities = g_PanelEntities.filter(panelEnt => panelEnts.find(ent => ent == panelEnt.ent));
 
 	for (let ent of panelEnts)
 	{
-		let panelEntState = GetExtendedEntityState(ent);
+		let panelEntState = GetEntityState(ent);
 		let template = GetTemplateData(panelEntState.template);
 
-		let panelEnt = g_PanelEntities.find(panelEnt => ent == panelEnt.ent);
+		let panelEnt = g_PanelEntities.find(pEnt => ent == pEnt.ent);
 
 		if (!panelEnt)
 		{
@@ -1029,7 +1104,7 @@ function displayPanelEntities()
 
 	buttons.forEach((button, slot) => {
 
-		if (button.hidden || g_PanelEntities.some(panelEnt => panelEnt.slot !== undefined && panelEnt.slot == slot))
+		if (button.hidden || g_PanelEntities.some(ent => ent.slot !== undefined && ent.slot == slot))
 			return;
 
 		button.hidden = true;
@@ -1075,8 +1150,7 @@ function updateGroups()
 	g_Groups.update();
 
 	// Determine the sum of the costs of a given template
-	let getCostSum = (ent) =>
-	{
+	let getCostSum = (ent) => {
 		let cost = GetTemplateData(GetEntityState(ent).template).cost;
 		return cost ? Object.keys(cost).map(key => cost[key]).reduce((sum, cur) => sum + cur) : 0;
 	};
@@ -1085,7 +1159,7 @@ function updateGroups()
 	{
 		Engine.GetGUIObjectByName("unitGroupLabel[" + i + "]").caption = i;
 
-		let button = Engine.GetGUIObjectByName("unitGroupButton["+i+"]");
+		let button = Engine.GetGUIObjectByName("unitGroupButton[" + i + "]");
 		button.hidden = g_Groups.groups[i].getTotalCount() == 0;
 		button.onpress = (function(i) { return function() { performGroup((Engine.HotkeyIsPressed("selection.add") ? "add" : "select"), i); }; })(i);
 		button.ondoublepress = (function(i) { return function() { performGroup("snap", i); }; })(i);
@@ -1120,20 +1194,20 @@ function updateDebug()
 
 	debug.hidden = false;
 
-	let conciseSimState = deepcopy(GetSimState());
+	let conciseSimState = clone(GetSimState());
 	conciseSimState.players = "<<<omitted>>>";
 	let text = "simulation: " + uneval(conciseSimState);
 
 	let selection = g_Selection.toList();
 	if (selection.length)
 	{
-		let entState = GetExtendedEntityState(selection[0]);
+		let entState = GetEntityState(selection[0]);
 		if (entState)
 		{
 			let template = GetTemplateData(entState.template);
 			text += "\n\nentity: {\n";
 			for (let k in entState)
-				text += "  "+k+":"+uneval(entState[k])+"\n";
+				text += "  " + k + ":" + uneval(entState[k]) + "\n";
 			text += "}\n\ntemplate: " + uneval(template);
 		}
 	}
@@ -1141,38 +1215,56 @@ function updateDebug()
 	debug.caption = text.replace(/\[/g, "\\[");
 }
 
-function getAllyStatTooltip(resource)
+/**
+ * Create ally player stat tooltip.
+ * @param {string} resource - Resource type, on which values will be sorted.
+ * @param {object} playerStates - Playerstates from players whos stats are viewed in the tooltip.
+ * @param {number} sort - 0 no order, -1 descending, 1 ascending order.
+ * @returns {string} Tooltip string.
+ */
+function getAllyStatTooltip(resource, playerStates, sort)
 {
-	let playersState = GetSimState().players;
-	let ret = "";
+	let tooltip = [];
 
-	for (let player in playersState)
-	{
-		if (player != 0 &&
-		    player != g_ViewedPlayer &&
-		    g_Players[player].state != "defeated" &&
-		    (g_IsObserver ||
-		       playersState[g_ViewedPlayer].hasSharedLos &&
-		       g_Players[player].isMutualAlly[g_ViewedPlayer]))
-		{
-			ret += "\n" + sprintf(translate("%(playername)s: %(statValue)s"),{
-				"playername": colorizePlayernameHelper("■", player) + " " + g_Players[player].name,
-				"statValue": resource == "pop" ?
-					sprintf(translate("%(popCount)s/%(popLimit)s/%(popMax)s"), playersState[player]) :
-					Math.round(playersState[player].resourceCounts[resource])
-			});
-		}
-	}
-
-	return ret;
+	for (let player in playerStates)
+		tooltip.push({
+			"playername": colorizePlayernameHelper("■", player) + " " + g_Players[player].name,
+			"statValue": resource == "pop" ?
+				sprintf(translate("%(popCount)s/%(popLimit)s/%(popMax)s"), playerStates[player]) :
+				Math.round(playerStates[player].resourceCounts[resource]),
+			"orderValue": resource == "pop" ? playerStates[player].popCount :
+				Math.round(playerStates[player].resourceCounts[resource])
+		});
+	if (sort)
+		tooltip.sort((a, b) => sort * (b.orderValue - a.orderValue));
+	return "\n" + tooltip.map(stat => sprintf(translate("%(playername)s: %(statValue)s"), stat)).join("\n");
 }
 
 function updatePlayerDisplay()
 {
-	let playerState = GetSimState().players[g_ViewedPlayer];
-	if (!playerState)
+	let allPlayerStates = GetSimState().players;
+	let viewedPlayerState = allPlayerStates[g_ViewedPlayer];
+	let viewablePlayerStates = {};
+	for (let player in allPlayerStates)
+		if (player != 0 &&
+			player != g_ViewedPlayer &&
+			g_Players[player].state != "defeated" &&
+			(g_IsObserver ||
+				viewedPlayerState.hasSharedLos &&
+				g_Players[player].isMutualAlly[g_ViewedPlayer]))
+			viewablePlayerStates[player] = allPlayerStates[player];
+
+	if (!viewedPlayerState)
 		return;
 
+	let tooltipSort = +Engine.ConfigDB_GetValue("user", "gui.session.respoptooltipsort");
+
+	let orderHotkeyTooltip = Object.keys(viewablePlayerStates).length <= 1 ? "" :
+		"\n" + sprintf(translate("%(order)s: %(hotkey)s to change order."), {
+		"hotkey": setStringTags("\\[Click]", g_HotkeyTags),
+		"order": tooltipSort == 0 ? translate("Unordered") : tooltipSort == 1 ? translate("Descending") : translate("Ascending")
+	});
+
 	let resCodes = g_ResourceData.GetCodes();
 	for (let r = 0; r < resCodes.length; ++r)
 	{
@@ -1189,19 +1281,20 @@ function updatePlayerDisplay()
 		if (descr)
 			tooltip += "\n" + translate(descr);
 
-		tooltip += getAllyStatTooltip(res);
+		tooltip += orderHotkeyTooltip + getAllyStatTooltip(res, viewablePlayerStates, tooltipSort);
 
 		resourceObj.tooltip = tooltip;
 
-		Engine.GetGUIObjectByName("resource["+r+"]_count").caption = Math.floor(playerState.resourceCounts[res]);
+		Engine.GetGUIObjectByName("resource[" + r + "]_count").caption = Math.floor(viewedPlayerState.resourceCounts[res]);
 	}
 
-	Engine.GetGUIObjectByName("resourcePop").caption = sprintf(translate("%(popCount)s/%(popLimit)s"), playerState);
+	Engine.GetGUIObjectByName("resourcePop").caption = sprintf(translate("%(popCount)s/%(popLimit)s"), viewedPlayerState);
 	Engine.GetGUIObjectByName("population").tooltip = translate("Population (current / limit)") + "\n" +
-		sprintf(translate("Maximum population: %(popCap)s"), { "popCap": playerState.popMax }) +
-		getAllyStatTooltip("pop");
+		sprintf(translate("Maximum population: %(popCap)s"), { "popCap": viewedPlayerState.popMax }) +
+		orderHotkeyTooltip +
+		getAllyStatTooltip("pop", viewablePlayerStates, tooltipSort);
 
-	g_IsTrainingBlocked = playerState.trainingBlocked;
+	g_IsTrainingBlocked = viewedPlayerState.trainingBlocked;
 }
 
 function selectAndMoveTo(ent)
@@ -1239,7 +1332,7 @@ function updateResearchDisplay()
 		if (numButtons >= 10)
 			break;
 
-		let template = GetTechnologyData(tech);
+		let template = GetTechnologyData(tech, g_Players[g_ViewedPlayer].civ);
 		let button = Engine.GetGUIObjectByName("researchStartedButton[" + numButtons + "]");
 		button.hidden = false;
 		button.tooltip = getEntityNames(template);
@@ -1255,6 +1348,9 @@ function updateResearchDisplay()
 		size.top = size.left + Math.round(researchStarted[tech].progress * (size.right - size.left));
 		Engine.GetGUIObjectByName("researchStartedProgressSlider[" + numButtons + "]").size = size;
 
+		Engine.GetGUIObjectByName("researchStartedTimeRemaining[" + numButtons + "]").caption =
+			Engine.FormatMillisecondsIntoDateStringGMT(researchStarted[tech].timeRemaining, translateWithContext("countdown format", "m:ss"));
+
 		++numButtons;
 	}
 
@@ -1291,7 +1387,8 @@ function recalculateStatusBarDisplay(remove = false)
 
 	Engine.GuiInterfaceCall("SetStatusBars", {
 		"entities": entities,
-		"enabled": g_ShowAllStatusBars && !remove
+		"enabled": g_ShowAllStatusBars && !remove,
+		"showRank": Engine.ConfigDB_GetValue("user", "gui.session.rankabovestatusbar") == "true"
 	});
 }
 
@@ -1302,14 +1399,13 @@ function recalculateStatusBarDisplay(remove = false)
 function toggleConfigBool(configName)
 {
 	let enabled = Engine.ConfigDB_GetValue("user", configName) != "true";
-	Engine.ConfigDB_CreateValue("user", configName, String(enabled));
-	Engine.ConfigDB_WriteValueToFile("user", configName, String(enabled), "config/user.cfg");
+	saveSettingAndWriteToUserConfig(configName, String(enabled));
 	return enabled;
 }
 
 /**
  * Toggles the display of range overlays of selected entities for the given range type.
- * @param {string} type - for example "Aura"
+ * @param {string} type - for example "Auras"
  */
 function toggleRangeOverlay(type)
 {
@@ -1330,6 +1426,15 @@ function toggleRangeOverlay(type)
 	});
 }
 
+function updateEnabledRangeOverlayTypes()
+{
+	for (let type of ["Attack", "Auras", "Heal"])
+		Engine.GuiInterfaceCall("EnableVisualRangeOverlayType", {
+			"type": type,
+			"enabled": Engine.ConfigDB_GetValue("user", "gui.session." + type.toLowerCase() + "range") == "true"
+		});
+}
+
 // Update the additional list of entities to be highlighted.
 function updateAdditionalHighlight()
 {
@@ -1340,7 +1445,6 @@ function updateAdditionalHighlight()
 		highlighted.push(g_Selection.highlighted[ent]);
 
 	if (g_ShowGuarding)
-	{
 		// flag the guarding entities to add in this additional highlight
 		for (let sel in g_Selection.selected)
 		{
@@ -1352,10 +1456,8 @@ function updateAdditionalHighlight()
 				if (highlighted.indexOf(ent) == -1 && entsAdd.indexOf(ent) == -1)
 					entsAdd.push(ent);
 		}
-	}
 
 	if (g_ShowGuarded)
-	{
 		// flag the guarded entities to add in this additional highlight
 		for (let sel in g_Selection.selected)
 		{
@@ -1366,7 +1468,6 @@ function updateAdditionalHighlight()
 			if (highlighted.indexOf(ent) == -1 && entsAdd.indexOf(ent) == -1)
 				entsAdd.push(ent);
 		}
-	}
 
 	// flag the entities to remove (from the previously added) from this additional highlight
 	for (let ent of g_AdditionalHighlight)
@@ -1445,7 +1546,7 @@ function sendLobbyPlayerlistUpdate()
 
 		let pData = g_GameAttributes.settings.PlayerData[playerID];
 
-		let minPData = { "Name": pData.Name };
+		let minPData = { "Name": pData.Name, "Civ": pData.Civ };
 
 		if (g_GameAttributes.settings.LockTeams)
 			minPData.Team = pData.Team;
@@ -1454,6 +1555,7 @@ function sendLobbyPlayerlistUpdate()
 		{
 			minPData.AI = pData.AI;
 			minPData.AIDiff = pData.AIDiff;
+			minPData.AIBehavior = pData.AIBehavior;
 		}
 
 		if (g_Players[playerID].offline)
@@ -1633,7 +1735,7 @@ function reportGame()
 
 		playerStatistics.economyScore += total + ",";
 		playerStatistics.militaryScore += Math.round((player.sequences.enemyUnitsKilledValue[maxIndex] +
-			player.sequences.enemyBuildingsDestroyedValue[maxIndex]) / 10)  + ",";
+			player.sequences.enemyBuildingsDestroyedValue[maxIndex]) / 10) + ",";
 		playerStatistics.totalScore += (total + Math.round((player.sequences.enemyUnitsKilledValue[maxIndex] +
 			player.sequences.enemyBuildingsDestroyedValue[maxIndex]) / 10)) + ",";
 
@@ -1657,24 +1759,23 @@ function reportGame()
 	reportObject.militaryScore = playerStatistics.militaryScore;
 	reportObject.totalScore = playerStatistics.totalScore;
 	for (let rct of resourcesCounterTypes)
-	{
 		for (let rt of resourcesTypes)
-			reportObject[rt+rct.substr(9)] = playerStatistics[rct][rt];
+			reportObject[rt + rct.substr(9)] = playerStatistics[rct][rt];
 			// eg. rt = food rct.substr = Gathered rct = resourcesGathered
-	}
+
 	reportObject.vegetarianFoodGathered = playerStatistics.resourcesGathered.vegetarianFood;
 	for (let type of unitsClasses)
 	{
 		// eg. type = Infantry (type.substr(0,1)).toLowerCase()+type.substr(1) = infantry
-		reportObject[(type.substr(0,1)).toLowerCase()+type.substr(1)+"UnitsTrained"] = playerStatistics.unitsTrained[type];
-		reportObject[(type.substr(0,1)).toLowerCase()+type.substr(1)+"UnitsLost"] = playerStatistics.unitsLost[type];
-		reportObject["enemy"+type+"UnitsKilled"] = playerStatistics.enemyUnitsKilled[type];
+		reportObject[(type.substr(0, 1)).toLowerCase() + type.substr(1) + "UnitsTrained"] = playerStatistics.unitsTrained[type];
+		reportObject[(type.substr(0, 1)).toLowerCase() + type.substr(1) + "UnitsLost"] = playerStatistics.unitsLost[type];
+		reportObject["enemy" + type + "UnitsKilled"] = playerStatistics.enemyUnitsKilled[type];
 	}
 	for (let type of buildingsClasses)
 	{
-		reportObject[(type.substr(0,1)).toLowerCase()+type.substr(1)+"BuildingsConstructed"] = playerStatistics.buildingsConstructed[type];
-		reportObject[(type.substr(0,1)).toLowerCase()+type.substr(1)+"BuildingsLost"] = playerStatistics.buildingsLost[type];
-		reportObject["enemy"+type+"BuildingsDestroyed"] = playerStatistics.enemyBuildingsDestroyed[type];
+		reportObject[(type.substr(0, 1)).toLowerCase() + type.substr(1) + "BuildingsConstructed"] = playerStatistics.buildingsConstructed[type];
+		reportObject[(type.substr(0, 1)).toLowerCase() + type.substr(1) + "BuildingsLost"] = playerStatistics.buildingsLost[type];
+		reportObject["enemy" + type + "BuildingsDestroyed"] = playerStatistics.enemyBuildingsDestroyed[type];
 	}
 	for (let type of misc)
 		reportObject[type] = playerStatistics[type];
